context("remove_pop")

dataDir <- system.file("extdata",package="flowWorkspaceData")
gs <- load_gs(list.files(dataDir, pattern = "gs_manual",full = TRUE))
gs1 <- clone(gs)
gs2 <- clone(gs)

test_that("remove_pop testing", {
  snapshot_pre <- getNodes(gs)
  add_pop(gs, alias = "*", pop = "+/-", parent = "/not debris", dims = "CD4", gating_method = "mindensity")
  snapshot_1 <- getNodes(gs)
  add_pop(gs, alias = "test_gate", parent = "/not debris/singlets", dims = "CD8", gating_method = "tailgate")
  snapshot_2 <- getNodes(gs)
  add_pop(gs, alias = "*", pop = "+/-", parent = "/not debris/singlets/CD3+/DNT", dims = "CCR7", 
          gating_method = "quantileGate", gating_args = "probs = 0.95")
  snapshot_3 <- getNodes(gs)
  remove_pop(gs)
  expect_equal(getNodes(gs), snapshot_2)
  remove_pop(gs)
  expect_equal(getNodes(gs), snapshot_1)
  remove_pop(gs)
  expect_equal(getNodes(gs), snapshot_pre)
  expect_error(remove_pop(gs))
  add_pop(gs, alias = "*", pop = "+/-", parent = "/not debris", dims = "CD4", gating_method = "mindensity")
  add_pop(gs, alias = "test_gate", parent = "/not debris/singlets", dims = "CD8", gating_method = "tailgate")
  add_pop(gs, alias = "*", pop = "+/-", parent = "/not debris/singlets/CD3+/DNT", dims = "CCR7", 
          gating_method = "quantileGate", gating_args = "probs = 0.98")
  expect_equal(getNodes(gs), snapshot_3)
  remove_pop(gs)
  remove_pop(gs)
  remove_pop(gs)

  # Just to prevent GatingSetList constructor from protesting overlapping samples
  sampleNames(gs2) <- "CytoTrol_Cytotrol_2.fcs"
  gslist <- GatingSetList(list(gs1, gs2))

  expect_equal(getNodes(gslist[[1]]), snapshot_pre)
  add_pop(gslist, alias = "*", pop = "+/-", parent = "/not debris", dims = "CD4", gating_method = "mindensity")
  expect_equal(getNodes(gslist[[1]]), snapshot_1)
  add_pop(gslist, alias = "test_gate", parent = "/not debris/singlets", dims = "CD8", gating_method = "tailgate")
  expect_equal(getNodes(gslist[[1]]), snapshot_2)
  add_pop(gslist, alias = "*", pop = "+/-", parent = "/not debris/singlets/CD3+/DNT", dims = "CCR7", 
          gating_method = "quantileGate", gating_args = "probs = 0.95")
  expect_equal(getNodes(gslist[[1]]), snapshot_3)
  remove_pop(gslist)
  expect_equal(getNodes(gslist[[2]]), snapshot_2)
  remove_pop(gslist)
  expect_equal(getNodes(gslist[[2]]), snapshot_1)
  remove_pop(gslist)
  expect_equal(getNodes(gslist[[2]]), snapshot_pre)
  expect_error(remove_pop(gslist))
  add_pop(gslist, alias = "*", pop = "+/-", parent = "/not debris", dims = "CD4", gating_method = "mindensity")
  add_pop(gslist, alias = "test_gate", parent = "/not debris/singlets", dims = "CD8", gating_method = "tailgate")
  add_pop(gslist, alias = "*", pop = "+/-", parent = "/not debris/singlets/CD3+/DNT", dims = "CCR7", 
          gating_method = "quantileGate", gating_args = "probs = 0.98")
  expect_equal(getNodes(gslist[[2]]), snapshot_3)
  remove_pop(gslist)
  remove_pop(gslist)
  remove_pop(gslist)
  
  # Repeat test for one of the component GatingSets to make sure it carried the adds and removals correctly
  expect_equal(getNodes(gs2), snapshot_pre)
  add_pop(gs2, alias = "*", pop = "+/-", parent = "/not debris", dims = "CD4", gating_method = "mindensity")
  expect_equal(getNodes(gs2), snapshot_1)
  add_pop(gs2, alias = "test_gate", parent = "/not debris/singlets", dims = "CD8", gating_method = "tailgate")
  expect_equal(getNodes(gs2), snapshot_2)
  add_pop(gs2, alias = "*", pop = "+/-", parent = "/not debris/singlets/CD3+/DNT", dims = "CCR7", 
          gating_method = "quantileGate", gating_args = "probs = 0.95")
  expect_equal(getNodes(gs2), snapshot_3)
  remove_pop(gs2)
  expect_equal(getNodes(gs2), snapshot_2)
  remove_pop(gs2)
  expect_equal(getNodes(gs2), snapshot_1)
  remove_pop(gs2)
  expect_equal(getNodes(gs2), snapshot_pre)
  expect_error(remove_pop(gs2))
  add_pop(gs2, alias = "*", pop = "+/-", parent = "/not debris", dims = "CD4", gating_method = "mindensity")
  add_pop(gs2, alias = "test_gate", parent = "/not debris/singlets", dims = "CD8", gating_method = "tailgate")
  add_pop(gs2, alias = "*", pop = "+/-", parent = "/not debris/singlets/CD3+/DNT", dims = "CCR7", 
          gating_method = "quantileGate", gating_args = "probs = 0.98")
  expect_equal(getNodes(gs2), snapshot_3)
  remove_pop(gs2)
  remove_pop(gs2)
  remove_pop(gs2)
  
  # Repeat test for loaded GatingSet with same guid to make sure starting from blank slate
  # after removals
  gs <- load_gs(list.files(dataDir, pattern = "gs_manual",full = TRUE))
  expect_equal(getNodes(gs), snapshot_pre)
  add_pop(gs, alias = "*", pop = "+/-", parent = "/not debris", dims = "CD4", gating_method = "mindensity")
  expect_equal(getNodes(gs), snapshot_1)
  add_pop(gs, alias = "test_gate", parent = "/not debris/singlets", dims = "CD8", gating_method = "tailgate")
  expect_equal(getNodes(gs), snapshot_2)
  add_pop(gs, alias = "*", pop = "+/-", parent = "/not debris/singlets/CD3+/DNT", dims = "CCR7", 
          gating_method = "quantileGate", gating_args = "probs = 0.95")
  expect_equal(getNodes(gs), snapshot_3)
  remove_pop(gs)
  expect_equal(getNodes(gs), snapshot_2)
  remove_pop(gs)
  expect_equal(getNodes(gs), snapshot_1)
  remove_pop(gs)
  expect_equal(getNodes(gs), snapshot_pre)
  add_pop(gs, alias = "*", pop = "+/-", parent = "/not debris", dims = "CD4", gating_method = "mindensity")
  add_pop(gs, alias = "test_gate", parent = "/not debris/singlets", dims = "CD8", gating_method = "tailgate")
  add_pop(gs, alias = "*", pop = "+/-", parent = "/not debris/singlets/CD3+/DNT", dims = "CCR7", 
          gating_method = "quantileGate", gating_args = "probs = 0.98")
  expect_equal(getNodes(gs), snapshot_3)

# Repeat test for loaded GatingSet with same guid to make sure starting from blank slate
# after add_pop_init()
  add_pop_init()
  gs <- load_gs(list.files(dataDir, pattern = "gs_manual",full = TRUE))
  expect_equal(getNodes(gs), snapshot_pre)
  add_pop(gs, alias = "*", pop = "+/-", parent = "/not debris", dims = "CD4", gating_method = "mindensity")
  expect_equal(getNodes(gs), snapshot_1)
  add_pop(gs, alias = "test_gate", parent = "/not debris/singlets", dims = "CD8", gating_method = "tailgate")
  expect_equal(getNodes(gs), snapshot_2)
  add_pop(gs, alias = "*", pop = "+/-", parent = "/not debris/singlets/CD3+/DNT", dims = "CCR7", 
          gating_method = "quantileGate", gating_args = "probs = 0.95")
  expect_equal(getNodes(gs), snapshot_3)
  remove_pop(gs)
  expect_equal(getNodes(gs), snapshot_2)
  remove_pop(gs)
  expect_equal(getNodes(gs), snapshot_1)
  remove_pop(gs)
  expect_equal(getNodes(gs), snapshot_pre)
  expect_error(remove_pop(gs))
  add_pop(gs, alias = "*", pop = "+/-", parent = "/not debris", dims = "CD4", gating_method = "mindensity")
  add_pop(gs, alias = "test_gate", parent = "/not debris/singlets", dims = "CD8", gating_method = "tailgate")
  add_pop(gs, alias = "*", pop = "+/-", parent = "/not debris/singlets/CD3+/DNT", dims = "CCR7", 
          gating_method = "quantileGate", gating_args = "probs = 0.98")
  expect_equal(getNodes(gs), snapshot_3)
})
